@@ -47,9 +47,25 @@ class EteSync:
         self._set_db(database)
 
     def _init_db_tables(self, database):
-        database.create_tables([pim.Content, cache.User, cache.JournalEntity,
+        database.create_tables([cache.Config, pim.Content, cache.User, cache.JournalEntity,
                                 cache.EntryEntity, cache.UserInfo], safe=True)
 
+        db_version = cache.Config.get_or_none()
+        if db_version is None:
+            from playhouse.migrate import SqliteMigrator, migrate
+            # Essentially version 0 so do first migration.
+            migrator = SqliteMigrator(database)
+
+            try:
+                migrate(
+                    migrator.add_column('journalentity', 'read_only', cache.JournalEntity.read_only),
+                )
+            except peewee.OperationalError:
+                # A hack because we don't have a db config yet.
+                pass
+
+            cache.Config.insert(db_version=1).execute()
+
     def get_or_create_user_info(self, force_fetch=False):
         user_info = None
         try:
@@ -102,9 +118,11 @@ class EteSync:
                 journal = existing[entry.uid]
                 del existing[journal.uid]
             else:
-                journal = cache.JournalEntity(local_user=self.user, version=entry.version, uid=entry.uid,
-                                              owner=entry.owner,
-                                              encrypted_key=entry.encrypted_key)
+                journal = cache.JournalEntity(local_user=self.user, uid=entry.uid, owner=entry.owner)
+
+            journal.version = entry.version
+            journal.encrypted_key = entry.encrypted_key
+            journal.read_only = entry.read_only
             journal.content = entry.getContent()
             journal.save()
 
@@ -126,7 +144,7 @@ class EteSync:
         changed = self._journal_list_dirty_get()
 
         for journal in changed:
-            crypto_manager = CryptoManager(journal.version, self.cipher_key, journal.uid.encode())
+            crypto_manager = self._get_journal_cryptomanager(journal)
             raw_journal = service.RawJournal(crypto_manager, uid=journal.uid)
             raw_journal.update(journal.content)
 
@@ -167,7 +185,7 @@ class EteSync:
         journal_uid = uid
         manager = EntryManager(self.remote, self.auth_token, journal_uid)
 
-        journal = cache.JournalEntity.get(uid=journal_uid)
+        journal = cache.JournalEntity.get(local_user=self.user, uid=journal_uid)
         crypto_manager = self._get_journal_cryptomanager(journal)
 
         collection = Journal._from_cache(journal).collection
@@ -188,7 +206,7 @@ class EteSync:
         return journal.content_set.where(pim.Content.new | pim.Content.dirty | pim.Content.deleted)
 
     def journal_is_dirty(self, uid):
-        journal = cache.JournalEntity.get(uid=uid)
+        journal = cache.JournalEntity.get(local_user=self.user, uid=uid)
         changed = list(self._journal_dirty_get(journal))
         return len(changed) > 0
 
@@ -197,8 +215,8 @@ class EteSync:
         journal_uid = uid
         manager = EntryManager(self.remote, self.auth_token, journal_uid)
 
-        journal = cache.JournalEntity.get(uid=journal_uid)
-        crypto_manager = CryptoManager(journal.version, self.cipher_key, journal_uid.encode())
+        journal = cache.JournalEntity.get(local_user=self.user, uid=journal_uid)
+        crypto_manager = self._get_journal_cryptomanager(journal)
         changed_set = self._journal_dirty_get(journal)
         changed = list(changed_set)
 
@@ -466,6 +484,10 @@ class Journal(ApiObjectBase):
     def version(self):
         return self._cache_obj.version
 
+    @property
+    def read_only(self):
+        return self._cache_obj.read_only
+
     @property
     def collection(self):
         journal_info = self.info
